//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//

#ifndef VKSiPMSensitiveDetector_h
#define VKSiPMSensitiveDetector_h 1

#include "VKSiPMHit.hh"

#include "G4DataVector.hh"
#include "G4VSensitiveDetector.hh"

#include <vector>

class G4Step;
class G4HCofThisEvent;

class VKSiPMSensitiveDetector : public G4VSensitiveDetector
{

  public:

    VKSiPMSensitiveDetector(G4String name);
    virtual ~VKSiPMSensitiveDetector();
 
    virtual void Initialize(G4HCofThisEvent* );

    virtual G4bool ProcessHits(G4Step* aStep, G4TouchableHistory* );
 
    // A version of processHits that does not modify aStep
    G4bool ProcessHits_constStep(const G4Step*, G4TouchableHistory* );

    virtual void EndOfEvent(G4HCofThisEvent* );
    virtual void clear();
    void DrawAll();
    void PrintAll();
 
    // Initialize the arrays to store SiPM positions
    void InitSiPMs(G4int nSiPMs);

    // Store a SiPM position
    void SetSiPMPositions(const std::vector<G4ThreeVector>& positions);

  private:
    G4int fHitsCID;
    VKSiPMHitsCollection* fSiPMHitCollection;   // <-- typedef in VKSiPMHit.hh

    G4DataVector* fSiPMPositionsX;
    G4DataVector* fSiPMPositionsY;
    G4DataVector* fSiPMPositionsZ;
};

#endif
