
#include <unittest++/UnitTest++.h>

#include "G4UImanager.hh"
#include "G4SystemOfUnits.hh"
#include "VKTabulatedField3D.hh"
#include "G4FieldManager.hh"
#include "G4TransportationManager.hh"


SUITE(VKBFieldTest) {

    class SetupFixture {
        public:
            SetupFixture()
            {
                // get pointer to UI manager
                UImanager = G4UImanager::GetUIpointer();

                // make sure magnetic field is enabled
                UImanager->ApplyCommand("/VK/field/enable true");
                                
                // get pointer to magnetic field
                field = static_cast<const VKTabulatedField3D*>
                    (G4TransportationManager::GetTransportationManager() 
                        -> GetFieldManager() -> GetDetectorField());                
            }
            const VKTabulatedField3D *field;
            G4UImanager *UImanager;
    };

    TEST_FIXTURE(SetupFixture, Sanity) {
        CHECK_EQUAL(1, 1);
    }
    
    TEST_FIXTURE(SetupFixture, TestMaxFieldStrength) {
        UImanager->ApplyCommand("/VK/field/enable true");
        UImanager->ApplyCommand("/VK/field/maxbfield 0.3 tesla");
        G4double b = field -> GetMaxFieldValue();
        CHECK_EQUAL(0.3, b/tesla);
    }    

    TEST_FIXTURE(SetupFixture, TestFieldIsModified) {
        const double point[4] = {0.02, -0.03, 0.07, 0.};   // x,y,z coordinates in meters
        double b1vec[3], b2vec[3];
        UImanager->ApplyCommand("/VK/field/enable true");
        UImanager->ApplyCommand("/VK/field/maxbfield 0.38 tesla");
        field -> GetFieldValue(point, b1vec);
        double b1 = sqrt(pow(b1vec[0], 2) + pow(b1vec[1], 2) + pow(b1vec[2], 2));
        UImanager->ApplyCommand("/VK/field/maxbfield 0.71 tesla");
        field -> GetFieldValue(point, b2vec);
        double b2 = sqrt(pow(b2vec[0], 2) + pow(b2vec[1], 2) + pow(b2vec[2], 2));
        CHECK_CLOSE(0.38/0.71, b1/b2, 1e-6);
    }    

    TEST_FIXTURE(SetupFixture, TestFieldCanBeDisabledAndEnabled) {
        // enable
        UImanager->ApplyCommand("/VK/field/enable true");
        bool fieldExists = G4TransportationManager::GetTransportationManager() 
                            -> GetFieldManager() -> DoesFieldExist(); 
        CHECK_EQUAL(true, fieldExists);
        // disable
        UImanager->ApplyCommand("/VK/field/enable false");
        fieldExists = G4TransportationManager::GetTransportationManager() 
                        -> GetFieldManager() -> DoesFieldExist(); 
        CHECK_EQUAL(false, fieldExists);
        // enable again, but with different field strength        
        UImanager->ApplyCommand("/VK/field/maxbfield 0.88 tesla");
        UImanager->ApplyCommand("/VK/field/enable true");
        fieldExists = G4TransportationManager::GetTransportationManager() 
                        -> GetFieldManager() -> DoesFieldExist(); 
        CHECK_EQUAL(true, fieldExists);
        // check that new field strength is correct
        const VKTabulatedField3D* newField = static_cast<const VKTabulatedField3D*>
                    (G4TransportationManager::GetTransportationManager() 
                        -> GetFieldManager() -> GetDetectorField()); 
        G4double b = newField -> GetMaxFieldValue();
        CHECK_EQUAL(0.88, b/tesla);
    }    
}
